function [w,y_class,acc]=multiclass_pairwise(data,y,v,method,M,params);
% This function calculates a multiple class classifier using the pairwise combination of classes. 
% Assumes that each data sample is independent and binary classes.
%
% Inputs:
%   data: matrix of traing/testing data, each row is a feature vector;
%   y: class indicator variable; assumes classes are labeled 1 through C.
%   v: number of partitions for the data
%   method: string giving learning method, 'percep', 'mse', 
%   (assumes linear model currently)
%   M: Number of classes
%   params: parameter vector for the selected classifier
%
%  Outputs:
%   params: learned parameters matrix, each row is a set of parameters from
%   one of the validation sets
%   err: error rate
%
%   Algorithm:
%   A. Separate out data for the appropriate binary subproblem
%   1. Divide up data into v training and testing sets
%   2. Train
%   3. Calculate error rate, repeat v times.
% Add a column of ones onto the data matrix for linear classifiers to allow for bias term
[N,nfeatures] = size(data);
data = [data,ones(N,1)];
err_test = zeros(M*(M-1)/2,v,1); 
w = zeros(M*(M-1)/2,nfeatures+1,v); % initialize weight values
funct_cnt = 0;

% start loop to go through all the classes pairwise
for nx = 1:M
    for ny = nx+1:M
        % convert class data to -1/+1 form for the selected set of classes
        ynew = (y==nx) - (y==ny);
        % remove data not in these two classes
        ind= find(ynew~=0);
        data_sub = data(ind,:);y_sub=ynew(ind);
        disp(['Class ',num2str(nx),' and Class ',num2str(ny)]);
 
        % Initialization
        [N,nfeatures] = size(data_sub);% size of updated data matrix
        disp(['Number of Feature Vectors ',num2str(N)]);
        funct_cnt = funct_cnt+1;
        
        % set up loop for cross validation

    
        for i=1:v % begin cross validation loop
    % 1. Divide data into v training and testing sets
            if (v>1)
                n_test = ceil(N/v);n_train = N - n_test;
                for i = 1:v
                    test_data = data_sub((i-1)*n_test+1:min(i*n_test,N),:);
                    test_y  = y_sub((i-1)*n_test+1:min(i*n_test,N));
                    if (i == 1)
                        train_data = data_sub(i*n_test+1:N,:);
                        train_y  = y_sub(i*n_test+1:N);
                    elseif (i==v)
                        train_data = data_sub(1:(v-1)*n_test,:);
                        train_y  = y_sub(1:(v-1)*n_test);
                    else
                        train_data = [data_sub(1:(v-1)*n_test,:);data_sub(i*n_test+1:N,:)];
                        train_y  = [y_sub(1:(v-1)*n_test);y_sub(i*n_test+1:N)];
                    end;% end if
                end; % end for loop
            else % if v =1 separate into 75% training, 25% test
                n_test = ceil(.25*N);n_train = N - n_test;
                train_data = data_sub(1:n_train,:);test_data = data_sub(n_train+1:N,:);
                train_y  = y_sub(1:n_train);test_y = y_sub(n_train+1:N);
            end;% end loop for training data separation
            %plotxval(train_data,test_data,train_y,test_y,[1,1,1]','check');
       % 2. Train the classifier
            w(funct_cnt,:,i) = feval(method,train_data,train_y,params)';
        
       % 3. Test classifier performance
            err_test(funct_cnt,i) = class_test(test_data, test_y, w(funct_cnt,:,i)');
       err_test(funct_cnt,i),
       % print results for this data set
            str =(['Classes ',num2str(nx),' and ',num2str(ny), method,' error cross validation set',num2str(i),' is ', num2str(err_test(funct_cnt,i))]);
            disp(str);
            plotxval(train_data,test_data,train_y,test_y,w(funct_cnt,:,i),str);
        end;% end cross validation loop
    end; % end inner class loop
end; % end outer class loop

% Testing phase
funct_cnt=1;
test_data = data;test_y=y;
[n_test,nfeat] = size(test_data);
test_out = zeros(M*(M-1)/2,n_test);
for nx=1:M
    for ny=(nx+1):M
        % test data against each classifier
        wtest=w(funct_cnt,:,1);
        test_out(funct_cnt,:) = (test_data*wtest')';%ignore cross-val right now
        ind = find(test_out(funct_cnt,:)>=0);
        test_out(funct_cnt,ind) = nx;
        ind = find(test_out(funct_cnt,:)<0);
        test_out(funct_cnt,ind) = ny; 
        funct_cnt = funct_cnt+1;
    end;
end;

class_cnts = histc(test_out,.5:M+.5);% find counts for each class
[mval,mind] = max(class_cnts);
y_class =mind';
acc = sum((y-y_class)==0)/length(y);
disp(['Accuracy ',num2str(100*acc),'% (',num2str(sum((y-y_class)==0)),'/',num2str(length(y)),')']);
